@@ -1,3 +1,5 @@
+__all__ = ['User']
+
 from dataclasses import dataclass
 
 from multibot import constants
@@ -15,4 +17,5 @@ class User(EventComponent):
     id: int = None
     name: str = None
     is_admin: bool = None
+    is_bot: bool = None
     original_object: constants.ORIGINAL_USER = None
